//+------------------------------------------------------------------+
//|                                                     mql4unit.mq4 |
//|                                      Copyright  2010 IT4Trading |
//|                                        http://www.it4trading.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010 IT4Trading"
#property link      "http://www.it4trading.com"
#property library

//-------------------------------------------------------------------+
//Current test conditions are kept by the global variables
//-------------------------------------------------------------------+
int tests_passed;    //Number of successful tests
int tests_failed;    //Number of unsuccessful tests
int tests_total;     //Total number of tests
string test_name;    //Test name

//-------------------------------------------------------------------+
//The function initializes test environment for one test
//-------------------------------------------------------------------+
void UnitTestStart(string TestName){
   test_name = TestName;
   tests_passed = 0;
   tests_failed = 0;
   tests_total = 0;
   Print("*--------------------------------------------------*");
   Print("Starting unit test execution ", test_name);
}

//-------------------------------------------------------------------+
//the function is called at the end of the test. Brings back true, if all the tests
//are successful. Otherwise - False.
//-------------------------------------------------------------------+
bool UnitTestEnd(){
   if (tests_failed == 0){
      Print("HURRAY!!! ", test_name, " PASSED. ", tests_passed, " tests are successful.");
   }else{
      Print(":((( ", test_name, " FAILED. ", tests_passed,"/",tests_total, " tests are successful.");   
   }
   Print("*--------------------------------------------------*");
}

//-------------------------------------------------------------------+
//The function executes the test for two arrays of int type
//Brings back true, if the arrays are equal
//-------------------------------------------------------------------+
bool TestIntArray(int actual[], int expected[]){
   tests_total++;
   //Comparing arrays' sizes
   if (ArraySize(actual) != ArraySize(expected)){
      Print("Test #", tests_total," ERROR. Array size ", ArraySize(actual), " instead of ", ArraySize(expected));
      tests_failed++;
      return(false);      
   }
   //Then comparing element by element
   for (int i=0; i<ArraySize(actual);i++){
      if (actual[i]!=expected[i]){
         Print("Test #", tests_total," ERROR. Element value #",i,"=", actual, " instead of ", expected);
         tests_failed++;
         return(false);
      }
   }
   //If all the elements are equal, the test is passed
   Print("Test #", tests_total," OK: Passed!");  
   tests_passed++;
   return(true);
}

//-------------------------------------------------------------------+
//The function executes the test for two arrays of double type
//Brings back true, if the arrays are equal
//-------------------------------------------------------------------+
bool TestDoubleArray(double actual[], double expected[], int precision){
   tests_total++;
   //Comparing arrays' sizes
   if (ArraySize(actual) != ArraySize(expected)){
      Print("Test #", tests_total," ERROR. Array size ", ArraySize(actual), " instead of ", ArraySize(expected));
      tests_failed++;
      return(false);      
   }
   //Then comparing element by element
   for (int i=0; i<ArraySize(actual);i++){
      if (NormalizeDouble(actual[i], precision)!=NormalizeDouble(expected[i], precision)){
         Print("Test #", tests_total," ERROR. Element value #",i,"=", DoubleToStr(actual, precision), " instead of ", DoubleToStr(expected, precision));
         tests_failed++;
         return(false);
      }
   }
   //If all the elements are equal, the test is passed
   Print("Test #", tests_total," OK: Passed!");  
   tests_passed++;
   return(true);
}

//-------------------------------------------------------------------+
//The function executes the test for two double type values
//Brings back true, if all the values within prescribed accuracy are equal
//-------------------------------------------------------------------+
bool TestDoubleValue(double actual, double expected, int precision){
   tests_total++;
   if (NormalizeDouble(actual,precision)==NormalizeDouble(expected,precision)){
      Print("Test #", tests_total," OK: Passed!");  
      tests_passed++;
      return(true);
   }else{
      Print("Test #", tests_total," ERROR. Value ", DoubleToStr(actual,precision), " instead of ", DoubleToStr(expected, precision));
      tests_failed++;
      return(false);
   }
}

//-------------------------------------------------------------------+
//The function executes the test for two int type values
//Brings back true, if the values are equal
//-------------------------------------------------------------------+
bool TestIntValue(int actual, int expected){
   tests_total++;
   if (actual==expected){
      Print("Test #", tests_total," OK: Passed!");  
      tests_passed++;
      return(true);
   }else{
      Print("Test #", tests_total," ERROR. Value ", actual, " instead of ", expected);
      tests_failed++;
      return(false);
   }
}


